/*
 * Copyright (c) 2020 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package varnish

import (
	"testing"

	"code.uplex.de/uplex-varnish/k8s-vcl-reloader/pkg/varnish/vcl"
)

func TestHasConfigMap(t *testing.T) {
	vc := &Controller{}
	cfgMap := vcl.ConfigMap{
		Name:    "vcl-cfgmap",
		Version: "47.11",
		Meta: vcl.Meta{
			Namespace:       "default",
			UID:             "d4b65c3f-c8d6-40eb-92cb-a3184c81dca2",
			ResourceVersion: "12345",
		},
	}
	if vc.HasConfigMap(cfgMap) {
		t.Errorf("HasConfigMap(%v) for nil spec, got:true, want:false",
			cfgMap)
	}

	vc.spec = &vcl.Spec{}
	vc.cfgLoaded = false
	if vc.HasConfigMap(cfgMap) {
		t.Errorf("HasConfigMap(%v) with cfgLoaded=false, "+
			"got:true, want:false", cfgMap)
	}

	vc.cfgLoaded = true
	vc.spec.CfgMap = vcl.ConfigMap{
		Name:    cfgMap.Name,
		Version: cfgMap.Version,
		Meta: vcl.Meta{
			Namespace:       cfgMap.Namespace,
			UID:             cfgMap.UID,
			ResourceVersion: cfgMap.ResourceVersion,
		},
	}
	if !vc.HasConfigMap(cfgMap) {
		t.Errorf("HasConfigMap(%v) with spec=%v, got:false, want:true",
			cfgMap, vc.spec)
	}

	vc.spec.CfgMap.Name = "foo"
	if vc.HasConfigMap(cfgMap) {
		t.Errorf("HasConfigMap(%v) with spec=%v, got:true, want:false",
			cfgMap, vc.spec)
	}

	vc.spec.CfgMap.Name = cfgMap.Name
	vc.spec.CfgMap.Version = "foo"
	if vc.HasConfigMap(cfgMap) {
		t.Errorf("HasConfigMap(%v) with spec=%v, got:true, want:false",
			cfgMap, vc.spec)
	}

	vc.spec.CfgMap.Version = cfgMap.Version
	vc.spec.CfgMap.Namespace = "foo"
	if vc.HasConfigMap(cfgMap) {
		t.Errorf("HasConfigMap(%v) with spec=%v, got:true, want:false",
			cfgMap, vc.spec)
	}

	vc.spec.CfgMap.Namespace = cfgMap.Namespace
	vc.spec.CfgMap.UID = "foo"
	if vc.HasConfigMap(cfgMap) {
		t.Errorf("HasConfigMap(%v) with spec=%v, got:true, want:false",
			cfgMap, vc.spec)
	}

	vc.spec.CfgMap.UID = cfgMap.UID
	vc.spec.CfgMap.ResourceVersion = "foo"
	if vc.HasConfigMap(cfgMap) {
		t.Errorf("HasConfigMap(%v) with spec=%v, got:true, want:false",
			cfgMap, vc.spec)
	}
}
